@if "%ECHO_ON%"=="" @echo off
:: --------------------------------------------------------------------------
:: This batch file applies a patch 
:: --------------------------------------------------------------------------
CLS 

:: Make sure our environment variable changes last only for this batch
:: (That way we do not have to backup individual env variables manually)
setlocal
set BATCHVERSION=V 01.05.00.03_02.01.00.01, 2021-06-16, Siemens AG
set WCFVERSION=2008
set WCF_ES_SHORT_VERSION="1.5.0.2"
set WCF_ES_VERSIONSTRING="2008 SP5 Upd2, Patch STEP 7 V5.7 Integrity"

:: --------------------------------------------------------------------------
:: Install only, if the right version of WinCC flexible is installed
:: --------------------------------------------------------------------------
set WCF_REQUIRED_ES_VERSION=2008 SP5 Upd2

:: --------------------------------------------------------------------------
:: Install only, if the right version of STEP 7 is installed
:: --------------------------------------------------------------------------
set WCF_REQUIRED_S7_VERSION=V5.7

:: --------------------------------------------------------------------------
:: Parse arguments
:: Arguments are listed in the form a=b
:: We parse the arguments after initialization by local setenv.bat or tools.ini
:: files to allow overrides.
:: Use a loop that shifts arguments instead of n for loops. That way we may
:: add arguments that in turn need arguments themselves such as "PATH xxyyzz".
:: Moreover we only need to go through all arguments once.
:: --------------------------------------------------------------------------
:: Check for missing command line parameters and show usage
set Apply_Uninstall=

if _%1==_ goto END_CHECKARGS

:ARGLOOP
    :: End argument loop if there are no more parameters
    if _%1==_ goto END_ARGLOOP

    :: Take current argument and remove surrounding quotation marks
	    set currentArg=%1
	    set currentArg=%currentArg:"=%
	    :: clear switches and values
	    set currentSwitch=&set currentValue=
	    for /F "tokens=1* delims=:" %%a in ("%currentArg%") do set currentSwitch=%%a
	    set currentValue=%currentArg:*:=%

		:: Enable uninstall mode
		if /i not %currentSwitch%==uninstall goto END_SWITCHCASE_SUPPLEMENT
			set Apply_Uninstall=1
			set UninstallPrefix=un
			goto END_SWITCHLOOP			
		:END_SWITCHCASE_SUPPLEMENT

	  :: This is an unknown argument
	    goto ERR_UNKNOWNARG

		:END_SWITCHLOOP
        goto END_ARGSWITCH
        
    :END_ARGSWITCH
    shift /1
    goto ARGLOOP
:END_ARGLOOP
:END_CHECKARGS

:: --------------------------------------------------------------------------
:: Startup message
:: --------------------------------------------------------------------------
echo.
echo [42;97m=========================================================================[0m
if defined Apply_Uninstall (
	echo [42;97m  WinCC flexible patch uninstaller                                       [0m
) else (
	echo [42;97m  WinCC flexible patch installer                                         [0m
)
echo [42;97m  %BATCHVERSION%                      [0m
echo [42;97m  %WCF_REQUIRED_ES_VERSION% %WCF_ES_SHORT_VERSION%                                                [0m
echo [42;97m=========================================================================[0m

:: ----------------------------------------------------
:: Set defaults 
:: ----------------------------------------------------
if not defined CmdRegEd set CmdRegEd=%windir%\regedit.exe
if not defined CmdRegEdOpts set CmdRegEdOpts=/e
if not defined CmdRegEdImportOpts set CmdRegEdImportOpts=/s
if not defined CopyCmd set CopyCmd=xcopy
if not defined CopyCmdFileOpts set CopyCmdFileOpts=/r /y /h /z /i
if not defined CopyCmdDirOpts set CopyCmdDirOpts=/s /e /i /r /y /h /z
if not defined DelCmdFile set DelCmdFile=del
if not defined DelCmdFileOpts set DelCmdFileOpts=/f /q
if not defined SimotionIntCmd set SimotionIntCmd=SimotionIntegrationSupport.exe
if not defined SimotionIntOpts set SimotionIntOpts=
if not defined CmdProcList set CmdProcList=pslist.exe
if not defined CmdFind set CmdFind=find
if not defined CmdFindOpts set CmdFindOpts=/I

set Product=WinCC flexible %WCFVERSION%

set PatchPathStart=%~dp0
set PatchDirStep7=Step7

:: ------------------------------------------------------------------------------
:: Issue progress message
echo.
echo [44;97m=========================================================================[0m
echo [44;97m                         Checking preconditions                          [0m
echo [44;97m=========================================================================[0m

:: ------------------------------------------------------------------------------
:: Check for admin rights
NET FILE 1>NUL 2>NUL
if NOT %errorlevel% == 0 goto :ERR_NOADMIN

:: ------------------------------------------------------------------------------
:: Check for path length
call :StringLength "%~dp0" strLenResult
if %strLenResult% GTR 145 goto :ERR_PATHTOOLONG

:: --------------------------------------------------------------------------
:: Continue initialization after command line evaluation
:: --------------------------------------------------------------------------
set StatusCode=0
set Apply_HandleSmartStart=1

SET OSBITS=32
IF /i "%PROCESSOR_ARCHITECTURE%" == "amd64" SET OSBITS=64
IF /i "%PROCESSOR_ARCHITEW6432%" == "amd64" SET OSBITS=64
echo.
echo Running on %OSBITS% bit OS

:: --------------------------------------------------------------------------
:: Check if patch has been installed before
if %OSBITS% == 64 (
  call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\%Product% Patch" OldHmiEsVersionString
) ELSE (
  call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\%Product% Patch" OldHmiEsVersionString
)
set OldHmiEsVersionString=%ReturnValue%

:: --------------------------------------------------------------------------
:: Get path of Product
:BEGIN_CHECK_PRODUCT
if %OSBITS% == 64 (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\AUTSW\%Product%" Path
) ELSE (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\AUTSW\%Product%" Path
)
set InstallPathHmies=%ReturnValue%
echo.
if "%InstallPathHmies%" == "" (
  echo - %Product% path not found.
  set Apply_HandleSmartStart=
  goto :END_CHECK_PRODUCT
)
echo - Found %Product% at:
echo     %InstallPathHmies%

:: --------------------------------------------------------------------------
:: Get version of Product
if %OSBITS% == 64 (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\AUTSW\%Product%" VersionString
) ELSE (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\AUTSW\%Product%" VersionString
)
set WCFVersionInstalled=%ReturnValue%
echo.
if "%WCFVersionInstalled%" == "" (
	echo - %Product% not found.
	set Apply_HandleSmartStart=
	goto :END_CHECK_PRODUCT
)
echo - Found WinCC flexible version:
echo     %WCFVersionInstalled%
:END_CHECK_PRODUCT

:: --------------------------------------------------------------------------
:: Get Step7 path
if %OSBITS% == 64 (
	call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\AUTSW\Step7" Path
) ELSE (
	call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\AUTSW\Step7" Path
)
set InstallPathStep7=%ReturnValue%
echo.
if "%InstallPathStep7%" NEQ "" goto :END_CHECK_STEP7_PATH_FOUND
if not defined Apply_Uninstall goto :END_CHECK_STEP7
:: When uninstalling and S7 is already uninstalled, try to get the path from our backup
if %OSBITS% == 64 (
  call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\%Product% Patch" S7InstallPath
) ELSE (
  call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\%Product% Patch" S7InstallPath
)
set InstallPathStep7=%ReturnValue%
if "%InstallPathStep7%" == "" (
	echo - STEP 7 path not found.
	goto :END_CHECK_STEP7
)
:END_CHECK_STEP7_PATH_FOUND
echo - Found STEP 7 at:
echo     %InstallPathStep7%

:: --------------------------------------------------------------------------
:: Get Step7 version
if %OSBITS% == 64 (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\AUTSW\STEP7" VersionString
) ELSE (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\AUTSW\STEP7" VersionString
)
set S7VersionInstalled=%ReturnValue%
echo.
if "%S7VersionInstalled%" == "" (
	echo - STEP 7 not found.
	if defined Apply_Uninstall goto :CHECK_STEP7_PATH
	goto :END_CHECK_STEP7
)
echo - Found STEP 7 version:
echo     %S7VersionInstalled%
:END_CHECK_STEP7

:: --------------------------------------------------------------------------
:: Get SIMOTION version
if %OSBITS% == 64 (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\AUTSW\UMC" VersionString 
) ELSE (
    call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\AUTSW\UMC" VersionString
)
set SimotionVersionInstalled=%ReturnValue%
echo.
if "%SimotionVersionInstalled%" == "" (
	echo - SIMOTION SCOUT not found.
	goto :END_CHECK_SIMOTION
)
echo - Found SIMOTION SCOUT version:
echo     %SimotionVersionInstalled%
set Apply_Simotion_Integration=1
:END_CHECK_SIMOTION

if not defined Apply_Uninstall (
	:: Check if paths really exist
	if "%WCFVersionInstalled%"=="" goto ERR_NOHMIES
	if not exist "%InstallPathHmies%" goto ERR_NOHMIPATH

	call :StartsWith "%WCFVersionInstalled%" "%WCF_REQUIRED_ES_VERSION%"
	if errorlevel 1 goto ERR_WRONGESVERSION	
	
	if "%S7VersionInstalled%"=="" goto ERR_NOS7
	if not exist "%InstallPathStep7%" goto :ERR_NOS7PATH

	call :StartsWith "%S7VersionInstalled%" "%WCF_REQUIRED_S7_VERSION%"
	if errorlevel 1 goto ERR_WRONGS7VERSION
	
	:: If WCF is not integrated, we cannot apply the S7 patch
	if not exist "%InstallPathStep7%\s7bin\UtahS7Integrate.ini" goto :ERR_ESNOTINTEGRATED
)

:: Check if STEP 7 is running
echo.
echo|set /p=- Check running state of STEP 7 ... 
"%PatchPathStart%%CmdProcList%" | %CmdFind% %CmdFindOpts% "S7tgtopx" > nul
if errorlevel 1 goto END_CHECKIFS7ISRUNNING
echo.
goto ERR_STEP7_IS_RUNNING
:END_CHECKIFS7ISRUNNING
echo not running

:: Check if SIMOTION SCOUT is running
echo.
echo|set /p=- Check running state of SIMOTION SCOUT ... 
"%PatchPathStart%%CmdProcList%" | %CmdFind% %CmdFindOpts% "u7wwbfax" > nul
if errorlevel 1 goto END_CHECKIFSIMOTIONISRUNNING
echo.
goto ERR_SIMOTION_IS_RUNNING
:END_CHECKIFSIMOTIONISRUNNING
echo not running

:: Pre-check if WinCC flexible is running
echo.
echo|set /p=- Check running state of WinCC flexible ... 
"%PatchPathStart%%CmdProcList%" | %CmdFind% %CmdFindOpts% "hmies" > nul
if errorlevel 1 goto END_CHECKIFHMIESISRUNNING
echo [33mrunning[0m
set HmiEsIsRunning=1
goto :END_CHECKHMIES
:END_CHECKIFHMIESISRUNNING
echo not running
:END_CHECKHMIES

if not defined Apply_HandleSmartStart goto :END_CHECKDOTNET40
if not defined HmiEsIsRunning goto :END_CHECKDOTNET40
:: Check if we have a .Net 4.0 available which is used to call HmiSmartStart.exe for shutting down HmiEs
if %OSBITS% == 64 (
	call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Microsoft\NET Framework Setup\NDP\v4\Client" Install
) ELSE (
	call :FUNC_GET_REG_ENTRY "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\NET Framework Setup\NDP\v4\Client" Install
)
if "%ReturnValue:~-1%"=="1" goto END_CHECKDOTNET40
goto ERR_DOTNET40_MISSING		
:END_CHECKDOTNET40

:: ------------------------------------------------------------------------------
:: Issue progress message
echo.
echo [44;97m=========================================================================[0m
if defined Apply_Uninstall (
	echo [44;97m               Uninstalling Patch STEP 7 V5.7 Integrity                  [0m
) else (
	echo [44;97m                Installing Patch STEP 7 V5.7 Integrity                   [0m
)
echo [44;97m=========================================================================[0m

set InstallPathBase=%InstallPathHmies:~0,-20%

:: ------------------------------------------------------------------------------
:: Check Step7
set PatchPath=%PatchPathStart%%PatchDirStep7%
echo.
echo Check STEP 7 installation...
call :StringLength "%PatchPath%" strLenResult
if %strLenResult% GTR 200 (
	goto :ERR_PATHTOOLONG
)

if not exist "%PatchPath%" goto END_STEP7_UPDATE

set InstallPathTarget=%InstallPathStep7%

if defined Apply_Uninstall (
	for /F %%i in ('dir /O-N /B %PatchPath%\s7mets') do (
		if exist "%InstallPathTarget%\s7mets\%%i" goto :END_STEP7_INSTALLCHECK
	)
	
	echo [33m - patch has already been uninstalled![0m
) else (
	for /F %%i in ('dir /O-N /B %PatchPath%\s7mets') do (
		if not exist "%InstallPathTarget%\s7mets\%%i" goto :END_STEP7_INSTALLCHECK
	)
	
	echo [33m - patch has already been installed![0m
)
goto :END_STEP7_UPDATE

:END_STEP7_INSTALLCHECK

if not defined HmiEsIsRunning goto END_STOP_SMART_START
:: Try to stop SmartStart and HmiEs and wait max 5 seconds
set RestartSmartStart=1
set ShutDownCounter=10
echo.
echo|set /p=Waiting for HmiSmartStart.exe to stop
"%InstallPathHmies%"\HmiSmartStart -shutdown

::wait 5 seconds for HmiEs to stop (triggered by HmiSmartStart)
:LOOP_STOP_SMART_START
:: wait 0.5 seconds (bug)
ping 127.0.0.1 -n 1 -w 2000 > nul
echo|set /p=.
set /a ShutDownCounter-=1

:: Check if WinCC flexible is still running
:CHECK_IF_HMIES_IS_RUNNING2
"%PatchPathStart%%CmdProcList%" | %CmdFind% %CmdFindOpts% "hmies" >nul
if errorlevel 1 goto END_CHECKIFHMIESISRUNNING2
if %ShutDownCounter% GTR 0 goto :LOOP_STOP_SMART_START
echo.
goto ERR_HMIES_IS_RUNNING
:END_CHECKIFHMIESISRUNNING2
echo.

:END_STOP_SMART_START

if defined Apply_Uninstall (
  echo|set /p =[33m - uninstalling...[0m
  call :FUNC_DELETE_FILE  "%InstallPathTarget%\s7mets\WCF_*met*"
) else (
  echo|set /p =[33m - installing...[0m
  call :FUNC_COPY_DIR "%PatchPath%\*.*" "%InstallPathTarget%"
)

:: Deleting catalog caches
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7met\S7ser.dat"
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7met\s7hCompressedMetaData.cmf"
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7met\S7cacheA.dat"
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7met\S7cacheB.dat"
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7mets\S7ser.dat"
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7mets\s7hCompressedMetaData.cmf"
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7mets\S7cacheA.dat"
call :FUNC_DELETE_FILE "%InstallPathTarget%\s7mets\S7cacheB.dat"

set InstallationChanged=1

echo [33m Done[0m

:END_STEP7_UPDATE

if not defined InstallationChanged goto :END_APPLY_SIMOTION_INTEGRATION
cd /d "%InstallPathHmies%"

:: ------------------------------------------------------------------------------
:: SIMOTION Integration
if not defined Apply_Simotion_Integration goto END_APPLY_SIMOTION_INTEGRATION
if not exist "%InstallPathHmies%\%SimotionIntCmd%" goto END_APPLY_SIMOTION_INTEGRATION
echo.
echo Recreating SIMOTION SCOUT Integration (please wait)...
call "%InstallPathHmies%\%SimotionIntCmd%" %SimotionIntOpts%
:END_APPLY_SIMOTION_INTEGRATION

:: ------------------------------------------------------------------------------
:: Finally update registry
set RegfileName=%TEMP%\UpdateRegistry.reg
echo REGEDIT4 >"%RegfileName%"
echo. >>"%RegfileName%"
::
:: set the "ES version info"

if "%WCFVersionInstalled%" == "" goto :SKIP_UPDATE_PRODUCT_REGISTRY_1
if %OSBITS% == 64 (
	echo [HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\AUTSW\%Product%] >>"%RegfileName%"
) ELSE (
	echo [HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\AUTSW\%Product%] >>"%RegfileName%"
)
if defined Apply_Uninstall (
	echo "VersionString"="%WCF_REQUIRED_ES_VERSION%" >>"%RegfileName%"
	echo "S7IntegrityMode"=- >>"%RegfileName%"
) else (
	if defined WCF_ES_VERSIONSTRING echo "VersionString"=%WCF_ES_VERSIONSTRING% >>"%RegfileName%"
	echo "S7IntegrityMode"="1" >>"%RegfileName%"
)
:SKIP_UPDATE_PRODUCT_REGISTRY_1

if defined Apply_Uninstall (	
	if %OSBITS% == 64 (
		echo [-HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\%Product% Patch] >>"%RegfileName%"
	) ELSE (
		echo [-HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\%Product% Patch] >>"%RegfileName%"
	)
	
	goto :END_UPDATE_REGISTRY
)

if %OSBITS% == 64 (
	echo [HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Siemens\%Product% Patch] >>"%RegfileName%"
) ELSE (
	echo [HKEY_LOCAL_MACHINE\SOFTWARE\Siemens\%Product% Patch] >>"%RegfileName%"
)
:: don't override version if patch was already installed before
if "%OldHmiEsVersionString%" NEQ "" goto :SKIP_SETHMIESVERSION
echo "OldHmiEsVersionString"="%WCFVersionInstalled%" >>"%RegfileName%"
:SKIP_SETHMIESVERSION
echo "S7InstallPath"="%InstallPathStep7:\=\\%" >>"%RegfileName%"

:END_UPDATE_REGISTRY

%CmdRegEd% %CmdRegEdImportOpts% "%RegfileName%"

if exist "%RegfileName%" del /q /f "%RegfileName%"

goto DONE

:: ==========================================================================
:: Subroutine: FUNC_GET_REG_ENTRY
::   param 1: The registry key
::   param 2: The name of the parameter to get
:: ==========================================================================
:FUNC_GET_REG_ENTRY
set RegfileNameSub=%TEMP%\CheckRegistry.reg
if exist "%RegfileNameSub%" del /q /f "%RegfileNameSub%" >nul
set RegKey=%1
%CmdRegEd% %CmdRegEdOpts% "%RegfileNameSub%" %RegKey%
set ReturnValue=
if not exist "%RegfileNameSub%" goto END_FUNC_GETREGENTRY_NOVALUE
type "%RegfileNameSub%" | %CmdFind% %CmdFindOpts% """%2""" >> NUL
if errorlevel 1 goto END_FUNC_GETREGENTRY_NOVALUE
FOR /F "usebackq tokens=1,2 delims==" %%a in (`type "%RegfileNameSub%" ^| %CmdFind% %CmdFindOpts% """%2"""`) do set ReturnValue=%%b
:: Unfortunately we have to do it twice for double double backslashes
set ReturnValue=%ReturnValue:\\=\%
set ReturnValue=%ReturnValue:\\=\%
set ReturnValue=%ReturnValue:"=%
:END_FUNC_GETREGENTRY_NOVALUE
if exist "%RegfileNameSub%" del /q /f "%RegfileNameSub%" >nul
goto :EOF

:: ==========================================================================
:: Subroutine: FUNC_DELETE_FILE
::   param 1: file to be deleted (including file mask)
:: ==========================================================================
:FUNC_DELETE_FILE
%DelCmdFile% %DelCmdFileOpts% %1 1>NUL 2>nul
goto :EOF

:: ==========================================================================
:: Subroutine: FUNC_COPY_DIR
::   param 1: source directory (including file mask)
::   param 2: destination directory
:: ==========================================================================
:FUNC_COPY_DIR
if not "%3"=="" goto FUNC_COPY_DIR_OUTPUT
%CopyCmd% %CopyCmdDirOpts% %1 %2 1>NUL 2>nul
goto :EOF
:FUNC_COPY_DIR_OUTPUT
%CopyCmd% %CopyCmdDirOpts% %1 %2
goto :EOF 

:: ==========================================================================
:: Subroutine: StartsWith
::   param 1: text to be searched
::   param 2: string to be tested for
::   Tests if a text starts with a given string
:: ==========================================================================
:StartsWith text string
SETLOCAL
set txt=%~1
set str=%~2
if defined str call set "s=%str%%%txt:*%str%=%%"
if /i "%txt%" NEQ "%s%" set=2>NUL
EXIT /b

:: ==========================================================================
:: Subroutine: StringLength
::   param 1: text to calculate the length for
::   param 1: variable to store the result
::   Returns the length of a given string
:: ==========================================================================
:StringLength string len
SetLocal EnableDelayedExpansion
set "token=#%~1" & set "len=0"
for /L %%A in (12,-1,0) do (
    set/A "len|=1<<%%A"
    for %%B in (!len!) do if "!token:~%%B,1!"=="" set/A "len&=~1<<%%A"
)
EndLocal & set %~2=%len%
EXIT /b

:ERR_NOADMIN
echo.
echo [41;93m*************************************************************************[0m
echo [41;93m*                                                                       *[0m
echo [41;93m*          Please run the patch with Administrator privilegues          *[0m
echo [41;93m*                                                                       *[0m
echo [41;93m*************************************************************************[0m
goto :LEVEL_1

:ERR_PATHTOOLONG
echo.
echo [31m*************************************************************************[0m
echo [31m*                                                                       *[0m
echo [31m*       The path where the patch is beeing executed is too long!        *[0m
echo [31m*                                                                       *[0m
echo [31m*            Please move the patch to C:\Temp and try again             *[0m
echo [31m*                                                                       *[0m
echo [31m*************************************************************************[0m
goto LEVEL_1

:ERR_UNKNOWNARG
echo. 
echo [31m !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31m ERROR: Found invalid command line argument '%1'.[0m
echo [31m                                                                         [0m	
echo [31m   Please use valid arguments only. Valid arguments are:                  [0m
echo [31m    uninstall                                                                 [0m
echo [31m                                                                         [0m	
echo [31m !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_NOHMIPATH
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! [0m
echo [31mERROR: Could not find WinCC flexible %WCFVERSION% path. [0m
echo [31m                                                                         [0m	
echo [31m  Please install WinCC flexible %WCFVERSION% before %UninstallPrefix%installing this patch.[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! [0m
goto LEVEL_1

:ERR_NOS7PATH
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! [0m
echo [31mERROR: Could not find STEP 7 path. [0m
echo [31m                                                                         [0m	
echo [31m  Please install STEP 7 V5.7 before %UninstallPrefix%installing this patch.[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! [0m
goto LEVEL_1

:ERR_NOHMIES
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31mERROR: Could not find WinCC flexible %WCFVERSION% executable. [0m
echo [31m                                                                         [0m	
echo [31m  Please install WinCC flexible %WCFVERSION% before %UninstallPrefix%installing this patch.[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_NOS7
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31m                                                                         [0m	
echo [31mERROR: STEP 7 is not installed. [0m
echo [31m  Please install STEP 7 V5.7 before %UninstallPrefix%installing this patch.[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_HMIES_IS_RUNNING
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31mERROR: WinCC flexible is still running![0m 
echo [31m                                                                         [0m	
echo [31m  This prevents the patch from being %UninstallPrefix%installed. Please close[0m 
echo [31m  WinCC flexible first before %UninstallPrefix%installing the patch.[0m 
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_STEP7_IS_RUNNING
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m 
echo [31mERROR: STEP 7 is still running![0m
echo [31m                                                                         [0m	
echo [31m  This prevents the patch from being %UninstallPrefix%installed. Please close[0m
echo [31m  STEP 7 first before %UninstallPrefix%installing the patch.[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_SIMOTION_IS_RUNNING
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m 
echo [31mERROR: SIMOTION SCOUT is still running![0m
echo [31m                                                                         [0m	
echo [31m  This prevents the patch from being %UninstallPrefix%installed. Please close[0m
echo [31m  SIMOTION SCOUT first before %UninstallPrefix%installing the patch.[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_DOTNET40_MISSING
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31mERROR: .Net Framework 4.0 is missing[0m
echo [31m                                                                         [0m	
echo [31m  Please install .Net 4.0 first, because it is[0m 
echo [31m  essential to have a fully working %Product%.[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_WRONGESVERSION
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31mERROR: Wrong version of WinCC flexible installed: WinCC flexible %WCFVersionInstalled%[0m
echo [31m                                                                         [0m	
echo [31m  This patch can only be installed on WinCC flexible %WCF_REQUIRED_ES_VERSION%[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_WRONGS7VERSION
echo. 
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31mERROR: Wrong version of STEP 7 installed: STEP 7 %S7VersionInstalled%[0m
echo [31m                                                                         [0m	
echo [31m  This patch can only be installed on STEP 7 %WCF_REQUIRED_S7_VERSION%[0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:ERR_ESNOTINTEGRATED
echo.
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
echo [31mERROR: WinCC flexible is not integrated into STEP 7                      [0m
echo [31m                                                                         [0m	
echo [31m Please start the Setup of WinCC flexible and enable the S7 integration, [0m
echo [31m before you install this patch.                                          [0m
echo [31m                                                                         [0m	
echo [31m!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!![0m
goto LEVEL_1

:: --------------------------------------------------------------------------
:: Here we are finished
:: --------------------------------------------------------------------------
:DONE

:: restart SmartStart
if not defined RestartSmartStart goto END_RESTART_SMART_START
echo.
echo Restart HmiSmartStart.exe
runas /trustlevel:0x20000 "%InstallPathHmies%\HmiSmartStart.exe /startup"
:END_RESTART_SMART_START

echo.
if %StatusCode% equ -1 (
	echo [41;93m=========================================================================[0m
	if defined Apply_Uninstall (
		echo [41;93m                Finished uninstalling patch with errors                  [0m
	) else (
		echo [41;93m                 Finished installing patch with errors                   [0m
	)
	echo [41;93m=========================================================================[0m
) else if %StatusCode% gtr 0 (
	echo [43;30m=========================================================================[0m
	if defined Apply_Uninstall (
  	echo [43;30m               Finished uninstalling patch with warnings                 [0m
	) else (
	  echo [43;30m                Finished installing patch with warnings                  [0m
	)
	echo [43;30m=========================================================================[0m
) else (
	echo [102;30m=========================================================================[0m
	if defined Apply_Uninstall (
	  echo [102;30m                     Successfully uninstalled patch                      [0m
	) else (
  	echo [102;30m                      Successfully installed patch                       [0m
	)
	echo [102;30m=========================================================================[0m
)

REM ==============================================================
REM Wait for key (Show results if invoked directly from explorer)
REM ==============================================================
:SHOW_RESULTS
echo.
pause
:END_SHOWRESULTS
endlocal
goto :EOF

:LEVEL_1
set StatusCode=-1
goto DONE

